--[[
	Addon communication plugin
]]--


--------------------------------------
-- Create plugin

local RW = LibStub("AceAddon-3.0"):GetAddon("Raid Watch")
local plugin = RW:Plugin("Comm", "AceComm-3.0", "AceSerializer-3.0")
if not plugin then return end

--------------------------------------
-- Locals
local prefix = "RW2"
local db
local throttle

--------------------------------------
-- Init

function plugin:OnRegister()
	self.db = RW.db:RegisterNamespace("Comm", {profile={}})
end

function plugin:PluginEnable()
	self:RegisterComm(prefix)
	self:RegisterMessage("RW_RegisterCommListener")
	self:RegisterMessage("RW_UnregisterAllCommListeners")
	
	self.listeners = {}
	throttle = {}
end

function plugin:PluginDisable()
	self.listeners = nil
	throttle = nil
end

--------------------------------------
-- Comm handling

local function handleCom(self, sender, dist, sucess, ...)
	if success then
		local listener = ...
		if self.listeners[listener] then
			for module in pairs(self.listeners[listener]) do
				if module[listener] and type(module[listener]) == "function" then
					module[listener](module, sender, select(2, ...))
				else
					module:OnSync(listener, sender, select(2, ...))
				end
			end
		end
	end
end

function plugin:OnCommReceived(_, message, dist, sender)
	handleCom(self, sender, dist, self:Deserialize(message))
end

--------------------------------------
-- Message handling

function plugin:RW_RegisterCommListener(message, module, listener)
	self.listeners[listener] = self.listeners[listener] or {}
	self.listeners[listener][module] = true
end

function plugin:RW_UnregisterListener(message, module, listener)
	if self.listeners[listener] then
		self.listener[listener][module] = nil
	end
end

function plugin:RW_UnregisterAllCommListeners(message, module)
	for listener, modules in pairs(self.listeners) do
		self.listeners[listener][module] = nil
	end
end

-- Send comm
function plugin:RW_SendComm(message, comMsg,  target, dist, ...)
	-- Throttle the sending of the same message one 2 seconds
	if not thottle[comMsg] or GetTime() - throttle[comMsg] > 2 then
		-- If no dist is proviced, RAID will be used over PARTY, if none of them are true then no message is sent
		dist = dist or (GetRealNumRaidMembers() > 0 and "RAID" or (GetRealNumPartyMembers() > 0 and "PARTY" or nil))
		if dist ~= nil then
			self:SendCommMessage(prefix, Self:Serialize(comMsg, ...), dist, target)
			throttle[comMsg] = GetTime()
		end
	end
end

---------------------------------------
-- Options

function plugin:GetOptions()
	local options = {
		type = "group",
		name = "Communication",
		args = {
		}
	}
	return options
end